#!/usr/bin/perl

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use CGI::EncryptForm;
use Email::Valid;
use File::Path;
use IO::Handle;
use GnuPG::Interface;
use GnuPG::Handles;
use Mail::SendEasy;
use DBI;
use Crypt::SMIME;
use Email::Send;
use Date::Format qw(time2str);
use String::SetUTF8;
use Digest::SHA1 qw(sha1_base64);

use integer;

$query = new CGI;

# The default main directory
my $maindir= '/privacybox/';


my $attachement_max_size= 60;

my @buchstaben= ('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z');

# Message translations
my %messages = (
     de => [ 'Session-Timeout, bitte loggen Sie sich erneut ein und wiederholen Sie den letzten Schritt.',
             'Bitte loggen Sie sich erneut ein und wiederholen Sie den letzten Schritt.',
             'Falsche Daten, bitte versuchen Sie es noch einmal.',
             'Account tempor&#228;r gesperrt, bitte warten Sie einen Moment.',
             'Unbekanntes Pseudonym oder falsches Passwort,<br>bitte versuchen Sie es noch einmal.',
             'Datenbankfehler, bitte kontaktieren Sie den Administrator.',
             'Logoutvorgang erfolgreich abgeschlossen!',
             'Die ausgew&#228;hlte Datei enth&#228;lt kein OpenPGP-Key oder S/MIME-Zertifikat im PEM-Format.',
             'Die angegebene E-Mail Adresse f&#252;r die Weiterleitung eingehender Nachrichten ist NICHT korrekt.',
             'Keine OpePGP-ID oder E-Mail Adresse angegeben, um den OpenPGP-Key zu identifizieren.',
             # 10
             'Fehler beim Test der OpenPGP-Verschl&#252;sselung! Haben Sie einen Public Key hochgeladen und stimmt die angegebene OpenPGP-ID?',
             'Kein S/MIME-Zertifikat f&#252;r Verschl&#252;sselung gefunden! Bitte w&#228;hlen Sie eine Datei mit dem Zertifikat im PEM-Format aus.',
             'Interner Ferhler, Nachrichten können nicht lokal gespeichert werden.',
             'Ferhler, Nachrichten können nicht an die angegebene E-Mail Adresse gesendet werden.',
             'Die Einstellungen wurden gepr&#252;ft und &#252;bernommen!<br><br>Eine Testnachricht wurde geschrieben. Bitte checken Sie Ihr Postfach.',
             'Datenfehler, die Aktualisierung wurde abgebrochen.',
             'Das Passwort f&#252;r den Login in das Webinterface und den POP3-Account wurde ge&#228;ndert!',
             'Die beiden neuen Passw&#246;rter sind nicht identisch!',
             'Falsches Passwort, bitte versuchen Sie es noch einmal.',
             'Die Kontaktinformation wurde gel&#246;scht!',
             # 20
             'Es ist keine Kontaktinformationen vorhanden!',
             'Die Kontaktinformation wurde aktualisiert.',
             'Interner Fehler, die Nachricht wurde nicht gespeichert!',
             'Die Nachricht f&#252;r das Admin-Team wurde gespeichert.',
             'Das Formular ist unvollst&#228;ndig ausgefüllt, bitte geben Sie einen Betreff und einen Text ein.',
             'Die Verschl&#252;sselung eingehender Nachrichten wurde deaktiviert, bitte pr&#252;fen Sie Ihre Einstellungen!.',
             'Ihr Account wurde <strong>gel&#246;scht</strong>!',
             'Angemeldet als:',
             'Einstellungen',
             'Daten &#228;ndern',
             # 30
             'Public Keys',
             'K&#252;ndigung',
             'Admin-Team',
             'Logout',
             'Beschreibung',
             'Anmelden',
             'Login',
             'Kontakt',
             'in Kooperation mit',
             'Pseudonym',
             # 40
             'Passwort',
             'Wenn Sie Fragen zu den Einstellungen haben, kontaktieren Sie das ',
             'Admin-Team',
             'Adressen Ihrer Kontaktseite',
             'Website',
             'Smartphone',
             'Hidden Service',
             'Weiterleitung eingehender Nachrichten',
             'Nachrichten auf diesem Server speichern und mit dem E-Mail Client abrufen.<br><span style="font-size:smaller;">(Empfohlen, wenn keine Verschl&#252;sselung genutzt wird.)</span>',
             'POP3-Server',
             # 50
             'Port',
             'Weiterleitung an externen E-Mail Account.<br><span style="font-size:smaller;">(Wird um eine zuf&#228;llige Zeit bis zu 1h verz&#246;gert, um einem globalen Beobachter wie z.B. Echelon<br>die Zuordnung von eingehendem und ausgehendem Traffic zu erschweren.)</span>',
             'E-Mail Adresse',
             'Verschl&#252;sselung eingehender Nachrichten',
             'keine Verschl&#252;sselung',
             'S/MIME Verschl&#252;sselung',
             'OpenPGP Verschl&#252;sselung',
             'OpenPGP-ID',
             'E-Mail Adresse oder 8-stellige ID: 0x????????',
             'Public Keys aktualisieren',
             # 60
             'OpenPGP-Key oder S/MIME-Zertifikat<br><span style="font-size:smaller;">(S/MIME-Zertifikate bitte im PEM-Format!)</span>',
             'Datei w&#228;hlen',
             'Einstellungen &#252;bernehmen und testen',
             'Passwort &#228;ndern',
             'altes Passwort',
             'neues Passwort',
             'Kontaktinformation &#228;ndern',
             'Kontaktadresse',
             'Empf&#228;nger',
             'Admin-Team der PrivacyBox',
             # 70
             'Betreff',
             'Nachricht / Anfrage senden',
             'S/MIME-Zertifikat',
             'Ein S/MIME-Zertifikat ist vorhanden.',
             'S/MIME-Zertifikat l&#246;schen',
             'OpenPGP-Schl&#252;ssel',
             'OpenPGP-Keys l&#246;schen',
             'Keine OpenPGP-Keys oder S/MIME-Zertifikate vorhanden.',
             'Mit der K&#252;ndigung werden alle zu ihrem Account vorhandenen Daten gel&#246;scht.<br>Es kann bis zu einer Stunde dauern, die Daten im Backup-System zu l&#246;schen.',
             'Account l&#246;schen',
             # 80
             'Invisible Internet',
             'Bevorzugte Sprache',
             'Weiterleitung an einen I2P-Mail Account.<br><span style="font-size:smaller;">(Keine Verschlüsselung mit S/MIME möglich.)</span>',
             'I2P Username',
             'Login auf IP-Adresse beschr&#228;nken',
             'Tor-Web-Proxy',
             'I2P-Web-Proxy',
             'Testnachricht'
           ],

     en => [ 'Timeout, please login again and retry the last step.',
             'Please login again and retry the last step.',
             'Wrong data sent, please try again.',
             'The account is locked temporary, please wait a moment.',
             'Wrong pseudonym or password please try again.',
             'Database error, please contact the admin.',
             'You are logged out!',
             'The file does not contain an OpenPGP key or S/MIME certificate in PEM format.',
             'The email address is not correct.',
             'Please set an OpenPGP id or an email address to select the OpenPGP key for encryption',
             # 10
             'OpenPGP encryption error! Did you upload an OpenPGP key and enter the correct OpenPGP id?',
             'No S/MIME certificate found! Please upload a certificate in PEM format.',
             'Internal error, unable to save messages local.',
             'Error: unable to send messages to this email address.',
             'Your settings were updated and a test message was send.<br><br>Please check your postbox and verify the encryption.',
             'Wrong data sent. Please contact the admin.',
             'Password changend.',
             'The new passwords are not identically.',
             'Wrong password, please try again.',
             'Contact information is removed.',
             # 20
             'No contact information found.',
             'Contact information updated.',
             'Internal error, the message was NOT saved.',
             'The message for the admin team was saved.',
             'Please write a subject and a message text.',
             'Encryption of messages is disabled! Please check your settings.',
             'Your account is <strong>deleted</strong>',
             'logged in as:',
             'account settings',
             'change data',
             # 30
             'public keys',
             'delete account',
             'admin team',
             'logout',
             'Description',
             'Sign up',
             'Login',
             'Contact',
             'in cooperation with',
             'pseudonym',
             # 40
             'password',
             'If there were questions about the settings, please ask the ',
             'admin team',
             'addresses of your contact page',
             'website',
             'mobile',
             'Tor hidden service',
             'forward of incoming messages',
             'save messages local and use the POP3 postbox with your email client<br><span style="font-size:smaller;">(recommended, particularly for non-encrypted messages)</span>',
             'pop3 server',
             # 50
             'port',
             'forward to an external email account',
             'email address',
             'encryption of incoming messages',
             'no encryption',
             'S/MIME encryption',
             'OpenPGP encryption',
             'OpenPGP key id',
             'email address or id of the OpenPGP key used for encryption',
             'upload public key',
             # 60
             'OpenPGP key or S/MIME certificate (PEM format)',
             'choose a key file',
             'check and update settings',
             'change password',
             'old password',
             'new password',
             'change contact information',
             'contact address',
             'recipient',
             'admin team of privacybox',
             # 70
             'subject',
             'send message',
             'S/MIME certificate',
             'S/MIME certificate is available.',
             'remove certificate',
             'OpenPGP keys',
             'remove all OpenPGP keys',
             'No OpenPGP keys or S/MIME certificates found.',
             'If your account was deleted, all data will be removed immediately.<br>It may take up to 1 hour, to remove all backup data.',
             'delete account',
             # 80
             'invisible internet',
             'Preferred language',
             'forward to an I2P mail account<br><span style="font-size:smaller;">(S/MIME encryption is NOT possible for this forward.)',
             'I2P user:',
             'enable IP-address check for this session',
             'Tor web proxy',
             'I2P web proxy',
             'test message'
           ],

     fr => [ 'D&eacute;lai d&apos;attente d&eacute;pass&eacute;. Veuillez s&apos;il vous pla&icirc;t vous reconnecter, puis r&eacute;it&eacute;rer l&apos;&eacute;tape pr&eacute;c&eacute;dente.',
             'Veuillez s&apos;il vous pla&icirc;t vous reconnecter, puis r&eacute;it&eacute;rer l&apos;&eacute;tape pr&eacute;c&eacute;dente.',
             'Les informations communiqu&eacute;es semblent erron&eacute;es. Veuillez s&apos;il vous pla&icirc;t r&eacute;essayer.',
             'Ce compte est temporairement bloqu&eacute;, veuillez s&apos;il vous pla&icirc;t patienter quelques minutes.',
             'Le pseudonyme ou le mot de passe sont erron&eacute;s, veuillez s&apos;il vous pla&icirc;t r&eacute;essayer.',
             'Erreur de la base de donn&eacute;es, veuillez s&apos;il vous pla&icirc;t prendre contact avec l&apos;administrateur syst&egrave;me.',
             'Vous &ecirc;tes maintenant d&eacute;connect&eacute; !',
             'Ce fichier ne contient aucune cl&eacute; OpenPGP ou certificat S/MIME en cours de validit&eacute;.',
             'Cette adresse de courrier &eacute;lectronique ne semble pas valide.',
             'Veuillez s&apos;il vous pla&icirc;t choisir un identifiant OpenPGP ou une adresse de courrier &eacute;lectronique, afin de s&eacute;lectionner la cl&eacute; OpenPGP qui permettra de chiffrer le contenu du message.',
             # 10
             'Erreur de chiffrement OpenPGP ! &ecirc;tes-vous certain d&apos;avoir pr&eacute;alablement t&eacute;l&eacute;charg&eacute; une cl&eacute; OpenPGP et renseign&eacute; l&apos;identifiant OpenPGP correspondant ?',
             'Le certificat S/MIME n&apos;a pu &ecirc;tre localis&eacute;. Veuillez s&apos;il vous pla&icirc;t t&eacute;l&eacute;charger un certificat au format PEM.',
             'Erreur interne, impossible de sauvegarder les messages en mode local.',
             'Erreur : impossible d&apos;envoyer les messages vers cette adresse de courrier &eacute;lectronique.',
             'Vos pr&eacute;f&eacute;rences ont &eacute;t&eacute; mises &agrave; jour, et un message de confirmation vous a &eacute;t&eacute; envoy&eacute;.<br><br>Veuillez s&apos;il vous pla&icirc;t contr&ocirc;ler le contenu de votre bo&icirc;te aux lettres et v&eacute;rifier le chiffrement.',
             'Les informations communiqu&eacute;es semblent erron&eacute;es. Veuillez s&apos;il vous pla&icirc;t prendre contact avec l&apos;administrateur syst&egrave;me.',
             'Ce mot de passe a &eacute;t&eacute; modifi&eacute;.',
             'Les nouveaux mots de passe ne correspondent pas.',
             'Ce mot de passe est erron&eacute;, veuillez s&apos;il vous pla&icirc;t r&eacute;essayer.',
             'Ces coordonn&eacute;es de contact ne sont plus accessibles.',
             'Aucune coordonn&eacute;e de contact ne correspond à votre demande.',
             'Les coordonn&eacute;es de contact ont &eacute;t&eacute; mises &agrave; jour.',
             'Erreur interne, le message n&apos;a PAS &eacute;t&eacute; sauvegard&eacute;.',
             'Le message &agrave; l&apos;&eacute;quipe des administrateurs a &eacute;t&eacute; sauvegard&eacute;.',
             'Veuillez s&apos;il vous pla&icirc;t d&eacute;finir l&apos;objet et renseigner le corps du message.',
             'Le chiffrement des messages est d&eacute;sactiv&eacute; ! Veuillez s&apos;il vous pla&icirc;t modifier vos pr&eacute;f&eacute;rences.',
             'Votre compte a &eacute;t&eacute; r&eacute;sili&eacute;.',
             'Connect&eacute; en tant que :',
             'Pr&eacute;f&eacute;rences',
             'Modifier les informations',
             'Cl&eacute;s publiques',
             'Supprimer le compte',
             'Contact',
             'D&eacute;connexion',
             'Description du service',
             'Inscription',
             'Connexion',
             'Contact',
             'Avec l&apos;aide de',
             'pseudonyme',
             'Mot de passe',
             'Si vous avez des questions &agrave; poser quant aux pr&eacute;f&eacute;rences, veuillez s&apos;il vous pla&icirc;t contacter le ',
             'Les administrateurs',
             'Adresse de votre page de contact',
             'Site Internet',
             'Mobile',
             'l&apos;interm&eacute;diaire du service TOR',
             'Redirection des messages entrants',
             'Sauvegarder les messages en mode local et utiliser la bo&icirc;te aux lettres POP3 avec votre client de courrier &eacute;lectronique (recommand&eacute;, en particulier pour les messages non chiffr&eacute;s).',
             'Serveur POP3',
             'Port',
             'Rediriger vers un compte de courrier &eacute;lectronique externe',
             'Adresse &eacute;lectronique',
             'Chiffrage des messages entrants',
             'Pas de chiffrement',
             'chiffrement S/MIME',
             'Chiffrement OpenPGP',
             'Identifiant de la cl&eacute; OpenPGP',
             'Adresse &eacute;lectronique ou identifiant correspondant<br>&agrave; la cl&eacute; OpenPGP utilis&eacute;e pour le chiffrement.',
             'T&eacute;l&eacute;charger la cl&eacute; publique',
             'Cl&eacute; OpenPGP ou certificat S/MIME<br><span style="font-size:smaller;">(pour les certificats S/MIME, veuillez utiliser le format PEM)</span>',
             'Veuillez choisir le fichier associ&eacute; &agrave; cette cl&eacute;',
             'Veuillez v&eacute;rifier et mettre &agrave; jour vos pr&eacute;f&eacute;rences',
             'Changer votre mot de passe',
             'Ancien mot de passe',
             'Nouveau mot de passe',
             'Changer vos coordonn&eacute;es de contact',
             'Adresse de contact',
             'd&eacute;stinataire',
             'Administrateurs de PrivacyBox',
             'subjet',
             'Envoyer le message',
             'Certificat S/MIME',
             'Le certificat S/MIME est valide',
             'Supprimer le certificat',
             'Cl&eacute;s OpenPGP',
             'Supprimer toutes les cl&eacute;s OpenPGP',
             'Aucune cl&eacute; OpenPGP ou certificat S/MIME en cours de validit&eacute; n&apos;a pu &ecirc;tre localis&eacute;.',
             'Si votre compte a &eacute;t&eacute; supprim&eacute;, la totalit&eacute; des donn&eacute;es aff&eacute;rentes sera imm&eacute;diatement effac&eacute;e. Le processus d&apos;effacement des donn&eacute;es de sauvegarde peut prendre jusqu&apos;&agrave; une heure.',
             'Supprimer le compte',
             'Le Projet Internet Furtif',
             'Choisissez votre langue',
             'Rediriger vers un compte de courrier &eacute;lectronique I2P<br><span style="font-size:smaller;">(S/MIME encryption is NOT possible for this.)',
             'I2P pseudonyme',
             'activer la v&eacute;rification de l&apos;adresse IP pour cette s&eacute;ance.',
             'proxy web TOR',
             'proxy web I2P',
             'Message de test'
           ]
     );

#################################
# Parameter einlesen
#################################


# outgoing smtp server
my $mailhost= 'localhost';
my $mailport= '25';
my $mailssl= 0;

# read the PBox main config file /etc/privacybox.conf, if exist
if (-e '/etc/privacybox.conf') {
	open (FILE, '/etc/privacybox.conf');
	my $line=0;
	while (<FILE>) {
		$line++;
		chomp;
		s/#.*//;
		s/^\s*//;
		s/\s*$//;
		next unless $_;

		my ($key, $value) = split(/\s+/, $_, 2);
		$key = lc($key);

		if($key eq 'pbox_main_dir') { $maindir= $value; }
	}
}

# Set the sub directories of the privacybox
if(!($maindir =~ /\/$/)) { $maindir= $maindir . '/'; }
my $main_dir= $maindir . 'cgi-data/';
my $mail_dir= $maindir . 'mail/';
my $admin_key_dir= $maindir . 'admin-keys';
my @admin_keys= undef;
my $cgi_conf= $maindir . 'cgi-conf/pbox.conf';

# define the params
my $form_encryption_passwd= undef;
my $mysql_user= undef;
my $mysql_passwd= undef;
my $self_host= undef;
my $onion_host= undef;
my $i2p_host= undef;
my $from_address= undef;
my $from_name= undef;
my $reply_address= undef;
my $reply_name = undef;

# read the params from config file
open (FILE, $cgi_conf);
my $line=0;
while (<FILE>) {
	$line++;
	chomp;
	s/#.*//;
	s/^\s*//;
	s/\s*$//;
	next unless $_;
	my ($key, $value) = split(/\s+/, $_, 2);
	$key = lc($key);
	next unless defined $value;

	if ($key eq 'host_name' ) {
		$self_host = $value;
	} elsif ($key eq 'onion_name' ) {
		$onion_host = $value;
	} elsif ($key eq 'i2p_name' ) {
		$i2p_host = $value;
	} elsif ($key eq 'user_conf_encrypt_pass' ) {
		$form_encryption_passwd = $value;
	} elsif ($key eq 'mysql_pass' ) {
		$mysql_passwd = $value;
	} elsif ($key eq 'mysql_user' ) {
		$mysql_user = $value;
	} elsif ($key eq 'pbox_from_address' ) {
		$from_address = $value;
	} elsif ($key eq 'pbox_from_name' ) {
		$from_name = $value;
	} elsif ($key eq 'pbox_reply_to_address' ) {
		$reply_address = $value;
	} elsif ($key eq 'pbox_reply_to_name' ) {
		$reply_name = $value;
	} elsif ($key eq 'admin_keys' ) {
		@admin_keys = split /,/, $value;
	}
}


#################################
# Vorbereitungen
#################################

# max size of a message in byte
my $max_message_size= ($attachement_max_size + 50) * 1024;
$CGI::POST_MAX = $max_message_size;

# the hostname of the server and URL of the script
my $conntyp= 0;
my $addr= $ENV{'REMOTE_ADDR'};
my $hostname= $ENV{'HTTP_HOST'};
if ( $hostname=~ /\.onion$/ ) { 
	$hostname= 'http://' . $hostname; 
	$conntyp= 11;
} elsif ( $hostname=~ /\.i2p$/ ) { 
	$hostname= 'http://' . $hostname; 
	$conntyp= 12;
} else { 
	$hostname= 'https://' . $hostname; 
	# check for JonDonym
	if (($addr eq '80.237.152.53')||($addr eq '141.76.45.35')||($addr eq '141.76.45.34')||
		($addr eq '212.227.103.74')||($addr eq '87.118.100.175')||($addr eq '80.237.191.141')||
		($addr eq '80.237.191.139')||($addr eq '87.230.56.32')||($addr eq '64.202.107.25')||
		($addr eq '193.164.133.197')||($addr eq '80.190.200.171')||($addr eq '87.230.56.74')||
		($addr eq '195.2.228.226')) { 
		$conntyp= 3;
	} else {
		$conntyp= 0;
	}
}
my $self= $hostname . $ENV{'SCRIPT_NAME'};

# bevorzugte Sprache ermitteln
my $lang= $query->param('lang');
if ((!(defined($lang)))  || ($lang eq "") || (!defined($messages{$lang}[0]))) { 
	my $tmplang= $ENV{'HTTP_ACCEPT_LANGUAGE'};
	if(defined($tmplang)) {
		if($tmplang =~ /^de/) { $lang= 'de'; }
		elsif($tmplang =~ /^fr/) { $lang= 'fr'; }
		else { $lang= 'en'; }
	} else { $lang= 'de'; }
}

# create form encyption object for later use
my $cfo = new CGI::EncryptForm(secret_key => $form_encryption_passwd);

# create some variables for later use
my $text_admin= '';
my $err_msg= undef;
my $info_msg= undef;
my $step_next= 0;
my $step_akt= 0;
my $step_akt_tmp= $query->param('ac');
if(defined($step_akt_tmp)) { $step_akt= $step_akt_tmp; }

# create variables for the login status
my $user_login= undef;
my $session_ip= undef;

###########################################
# check the login session status
###########################################
my $session= $query->param('se');
if(defined($session)) { 
	my $newhashref = $cfo->decrypt($session);
	my $user_login_tmp= $newhashref->{'uid'};
	my $sesson_id= $newhashref->{'sid'};
	my $sessionfile= $main_dir . $user_login_tmp . '/session.txt';
	if( -f $sessionfile ) {
		open(INFILE, $sessionfile );
		if (INFILE) {
			my $session_config= <INFILE>;
			chomp $session_config;
			close INFILE;

			my $sessionhashref = $cfo->decrypt($session_config);
			$session_ip= $sessionhashref->{'ip'};
			my $old_time= $sessionhashref->{'lts'};
			my $old_id= $sessionhashref->{'sid'};
			if ((defined($old_id)) && ($old_id == $sesson_id) && (defined($old_time)) && (time() - $old_time < 1800)) {
				if(defined($session_ip)) {
					# check IP hash, if defined one
					my $ip_digest = sha1_base64($addr);
					if ($ip_digest eq $session_ip) { 
						$user_login= $user_login_tmp; 
					}
					else {
						 # IP change, logout the user!
						$step_akt= 101; 
						$err_msg= $messages{$lang}[0];
						system("shred -n 2 -u $sessionfile");
					}
				} else { $user_login= $user_login_tmp; }
			} else { 
				$step_akt= 101; 
				$err_msg= $messages{$lang}[0];
				system("shred -n 2 -u $sessionfile"); 
			}
		} else { 
			$step_akt= 101; 
			$err_msg= $messages{$lang}[1];
		}
	} else { 
		$step_akt= 101; 
		$err_msg= $messages{$lang}[1];
	}
}

###########################################################
# Working part of the script
#
# depending on $step_akt:
# 1: Login
# 2: Logout
# 5: update config values
# 9: change password
# 11: change contact address
# 13: send to admin team
# 23: Account löschen
###########################################################
if ( $step_akt == 1 ) {
	# user send login data, check the user_id/password
	$user_login= undef;
	my $user_tmp= $query->param('us');
	my $passwd= $query->param('pw');
	if((!defined($passwd)) || ($passwd eq "") || (!defined($user_tmp)) || ($user_tmp eq "") || 
		(! ($user_tmp =~ /^[a-zA-Z0-9\-\._]+$/))) { 
		$step_next= 101; 
		$err_msg= $messages{$lang}[2];
	} else {
		my $user_dir= $main_dir . $user_tmp;
		if (-d $user_dir) {
			my $mysql = DBI -> connect ("DBI:mysql:database=privacybox", $mysql_user, $mysql_passwd, {'RaiseError' => 1});
			my $statement = $mysql -> prepare ("SELECT * FROM passwd WHERE username = ?" );
			$statement -> execute ($user_tmp);
			if (my $row = $statement -> fetchrow_hashref ()) {
				my $passwd_saved= $row -> {'passwd_crypt'};
				$statement->finish ();
				$mysql->disconnect ();
				my @pa= split //, $passwd_saved;
				my $salt2= $pa[0] . $pa[1];
				my $user_lock= $user_dir . '/lock.txt';
				if( -f $user_lock ) { 
					$step_next= 101; 
					$err_msg= $messages{$lang}[3];
				} else {
					# check the password
					if (crypt($passwd, $salt2) eq $passwd_saved) {
						# everything is ok, change to the new user andprint settings
						$user_login= $user_tmp;
						$step_next= 4;
						# choose the preferred language
						my $tconfig= $main_dir . $user_login . '/user.cfg';
						open(CFG, $tconfig);
						my $user_configuration= <CFG>;
						chomp $user_configuration;
						close CFG;
						my $newhashref = $cfo->decrypt($user_configuration);
						if (defined($newhashref->{'la'})) { $lang= $newhashref->{'la'}; }

						# check for IP address check
						my $user_ip_check= $query->param('ip');
						if((defined($user_ip_check)) && ($user_ip_check eq '1')) {
							$session_ip = sha1_base64($addr);
						}
					} else { 
            # lock the account for 6sec
						open(LCK, ">$user_lock");
						print LCK "!\n";
						close LCK;
						$step_next= 0; 
						$err_msg= $messages{$lang}[4];
						sleep 6;
						unlink $user_lock;
					}
				}
			} else {
				$mysql -> disconnect ();
				$step_next= 101; 
				$err_msg= $messages{$lang}[5];
			}
		} else { 
			sleep 4;
			$step_next= 101;
			$err_msg= $messages{$lang}[4];
		}
	}
}

elsif ( $step_akt == 2 ) {
	# logout the current user
	my $sessionfile= $main_dir . $user_login . '/session.txt';
	if( -f $sessionfile ) { system("shred -n 2 -u $sessionfile"); }
	$user_login= undef;
	$session_ip= undef;
	$step_next= 100; 
	$info_msg= $messages{$lang}[6];
}

elsif ( $step_akt == 5 ) {
	# get all form params
	my $forward= $query->param('fw');
	my $email= $query->param('em');
	my $encryption= $query->param('encr');
	my $pgp_id= $query->param('pgp');
	my $keyfile= $query->param('key');
	my $i2p_reci= $query->param('i2p');
	$lang= $query->param('la');
	if (($lang ne 'de') && ($lang ne 'en') && ($lang ne 'fr')) { $lang= 'en'; }

	if((defined($forward)) && (defined($email)) && (defined($encryption)) && (defined($keyfile)) && (defined($pgp_id))
		&& (defined($i2p_reci))) {
		if ($keyfile ne "") {
			# upload the new keyfile
			my $anhang= $query->upload('key');
			my $line= <$anhang>;
			chomp $line;
			if ($line =~ /BEGIN PGP PUBLIC KEY BLOCK/) {
				# OpenPGP key upload
				my $user_pgp_file= $main_dir . $user_login . '/openpgp.asc';
				open DAT, ">$user_pgp_file";
				print DAT $line, "\n";
				$line= <$anhang>;
				chomp $line;
				if(!( $line =~ /^Version:/ )) { print DAT $line, "\n"; }
				while($line= <$anhang>) {
					chomp $line;
					print DAT $line, "\n";
				}
				close DAT;

				# delete the minimzed OpenPGP key if exits
				my $file= $main_dir . $user_login . '/openpgp_mini.asc';
				if(-f $file) { system("shred -n 2 -u $file"); }

				# import the PGP-Key to keyring
				my $gnupgpath= $main_dir . $user_login;
				my $gnupgd= GnuPG::Interface->new();
				$gnupgd->options->hash_init( armor   => 1, homedir => $gnupgpath, always_trust => 1 );
				$gnupgd->options->meta_interactive(0);
				my $reind= IO::Handle->new();
				my $rausd= IO::Handle->new();
				my $rerrd= IO::Handle->new();
				my $rrd= GnuPG::Handles->new(stdin => $reind, stdout => $rausd, stderr => $rerrd);
				$gpgpid = $gnupgd->wrap_call( 
					commands => [ qw(--import) ],
					command_args =>  $user_pgp_file ,
					handles => $rrd 
				);
				waitpid $gpgpid, 0;
				# my @gpg_err_msg= <$rerrd>;
				close $reind;
				close $rausd;
				close $rerrd;

			} elsif ($line =~ /BEGIN CERTIFICATE/) {
				# S/MIME certificate upload
				my $user_smime_file= $main_dir . $user_login . '/smime_tmp.pem';
				open DAT, ">$user_smime_file";
				print DAT $line, "\n";
				while($line= <$anhang>) {
					chomp $line;
					print DAT $line, "\n";
				}
				close DAT;
			} else {
				$step_next= 4;
				$err_msg= $messages{$lang}[7];
			}
		}
		if ($step_next == 0) {
			# validate forward email address
			if ($forward eq '2') {
				if (!(Email::Valid->address($email) )) {
					$step_next= 4;
					$err_msg= $messages{$lang}[8];
				}
			}
		}
		if ($step_next == 0) {
			# validate encryption
			my $text;
			if($lang eq 'de') {
				$text= "Diese Nachricht ist ein Test.\n\nBitte prüfen Sie die die Einstellungen\nfür Verschlüsselung und Weiterleitung.\n\nDer Postmaster";
			} elsif ($lang eq 'fr') {
				$text= "Ceci est un message de test.\n\nMerci de valider vos préférences relatives au chiffrement et au transfert de vos mails.\n\nLe Postmaster de PrivacyBox";
			} else {
				$text= "This is only a test message.\n\nPlease validate your settings for encryption and forwarding.\n\nPostmaster of the PrivacyBox";
			}
			my $message= 'From: ' . $from_name . ' <' . $from_address . '>' . "\n";
			my $datum= time2str("%a, %e %b %Y %T %z", time() );
			$message= $message . "Date: $datum\n";
			$message= $message . "User-Agent: PrivacyBox 1.0\nMIME-Version: 1.0\n";
			$message= $message . "Content-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\n";
			$message= $message . "Subject: " . $messages{$lang}[87] . "\n"; 
			if ($forward eq '2') { 
				$message= $message . "To: $email\n"; 
				$message= $message . 'Reply-to: ' . $reply_name . ' <' . $reply_address . '>' .   "\n\n";
			}
			else { $message= $message . 'To: ' . $user_login . '@' . $self_host . "\n\n"; }

			if ($encryption eq '1') {
				# test openpgp encryption
				if($pgp_id eq '') {
					if($email eq '') {
						$step_next= 4;
						$err_msg= $messages{$lang}[9];
					} else { $pgp_id= $email; }
				} 
				if($step_next == 0) {
					my $gnupgpath= $main_dir . $user_login;
					my $gnupg= GnuPG::Interface->new();
					$gnupg->options->hash_init( armor   => 1, homedir => $gnupgpath, 
						always_trust => 1, extra_args => '--no-emit-version' );
					$gnupg->options->push_recipients( $pgp_id ); 
					$gnupg->options->meta_interactive(0);

					my $rein= IO::Handle->new();
					my $raus= IO::Handle->new();
					my $rerr= IO::Handle->new();
					my $rr= GnuPG::Handles->new(stdin => $rein, stdout => $raus, stderr => $rerr );

					my $gpg_pid= $gnupg->encrypt( handles => $rr );
					if ($forward eq "3") { setUTF8($text); }
					print $rein $text;
					close $rein;
					my @errortext;
					if($rerr) { @errortext= <$rerr>; }
					my $cipherline= '';
					my $ciphertext= '';
					if(defined($raus)) {
						while ($cipherline= <$raus>) { 
							$message= $message . $cipherline; 
							$ciphertext= $ciphertext . $cipherline;
						}
					}
					close $raus;
					close $rerr;
					waitpid $gpg_pid, 0;
					if ( @errortext > 0) {
						$step_next= 4;
						$err_msg= $messages{$lang}[10];
					} else {
						$text= $ciphertext;
					}
				}
			} elsif ($encryption eq '2') {
				# Test S/MIME encryption
				my $smime_file= $main_dir . $user_login . '/smime_tmp.pem';
				if (!(-r $smime_file)) {
					$step_next= 4;
					$err_msg= $messages{$lang}[11];
				} else {
					my $smime_cert;
					open(CERT, $smime_file);
					while(<CERT>) { $smime_cert= $smime_cert . $_; }
					close CERT;
					my $message_unencr= $message . $text;
					my $smime = Crypt::SMIME->new();
					$smime->setPublicKey($smime_cert);
					$message= $smime->encrypt($message_unencr);
					$smime_file= $main_dir . $user_login . '/smime.pem';
					open(CERT, ">$smime_file");
					print CERT $smime_cert;
					close CERT;
				}
			} else {
				# no encryption, send simple test mail
				$message= $message . $text;
			}
			if ($step_next == 0) {
				# Senden der Testmail
				if ($forward eq "1") {
					# save message local
					my $random_msg= int(rand 120000);
					my $mailmsg= $mail_dir . $user_login . '/Maildir/new/' . $random_msg . '@' . $self_host;
					while(-f $mailmsg) {
						$random_msg= int(rand 120000);
						$mailmsg= $mail_dir . $user_login . '/Maildir/new/' . $random_msg . '@' . $self_host;
					}
					open(MSG, ">$mailmsg");
					if (MSG) {
						print MSG $message;
						close MSG;
					} else { 
						$step_next= 4;
						$err_msg= $messages{$lang}[12];
					}
				} elsif ($forward eq "2") {
					# send message to an extern mail account
					my $mailer = Email::Send->new({mailer => 'SMTP'});
					$mailer->mailer_args([Host => "$mailhost:$mailport", ssl => $mailssl]);
					if (! $mailer->send($message)) {
						$step_next= 4;
						$err_msg= $messages{$lang}[13];
					}
				} else {
					# send message to an ip2 mail account
					if($encryption ne '1') { setUTF8($text); }
					my $mail = new Mail::SendEasy( 
						smtp => 'localhost' , 
						port => '7659', 
						user => $i2p_user, 
						pass => $i2p_pass,
						timeout => 250 
					);
					my $status = $mail->send(
						from    => $i2p_user . '@mail.i2p' ,
						from_title => $from_name ,
						to      => $i2p_reci . '@mail.i2p' ,
						subject => $messages{$lang}[87] ,
						msg     => $text
					) ;
					if (!$status) { 
						$error= 4;
						$err_msg= $messages{$lang}[13];
					}
				}
			}
		}
		if ($step_next == 0) {
			# take the new settings
			my $ahashref = { 
				encr => $encryption,
				fwd => $forward,
				pgp => $pgp_id,
				email => $email,
				la => $lang,
				i2p => $i2p_reci 
			};
			my $save= $cfo->encrypt($ahashref);
			my $configfile= $main_dir . $user_login . '/user.cfg';
			open(OUTF, ">$configfile");
			print OUTF $save, "\n";
			close OUTF;

			$step_next= 100;
			$info_msg= $messages{$lang}[14];
		}
	} else {
		$step_next= 100;
		$err_msg= $messages{$lang}[15];
	}
}

elsif ( $step_akt == 9 ) {
	# user send the change password request
	if (defined($user_login)) { 
		my $passwd= $query->param('pw0');
		my $passwd1= $query->param('pw1');
		my $passwd2= $query->param('pw2');
		if((defined($passwd)) && (defined($passwd1)) && (defined($passwd2)) &&
			($passwd ne '') && ($passwd1 ne '') && ($passwd2 ne '')) {
			# read and compare the old password
			my $mysql = DBI -> connect ("DBI:mysql:database=privacybox", $mysql_user, $mysql_passwd, {'RaiseError' => 1});
			my $statement = $mysql -> prepare ("SELECT * FROM passwd WHERE username = ?");
			$statement -> execute ($user_login);
			if (my $row = $statement -> fetchrow_hashref ()) {
				my $passwd_saved= $row -> {'passwd_crypt'};
				$statement->finish ();

				my @pa= split //, $passwd_saved;
				my $salt2= $pa[0] . $pa[1];
				my $passwd_crypt= crypt($passwd, $salt2);
				if(  $passwd_crypt eq $passwd_saved) {
					if($passwd1 eq $passwd2) {
						# everything is ok, set the new password
						$salt2= $buchstaben[int(rand @buchstaben)] . $buchstaben[int(rand @buchstaben)];
						$passwd_crypt= crypt($passwd1, $salt2);
						$statement = $mysql -> prepare ("UPDATE passwd SET passwd_crypt = ? WHERE username = ?");
						$statement -> execute ($passwd_crypt, $user_login);
						$statement->finish ();
						$info_msg= $messages{$lang}[16];
						$step_next= 100; 
					} else {
            # new passwords does not match
						$step_next= 8; 
						$err_msg= $messages{$lang}[17]; 
					}
				} else { 
					# wrong old password, lock account for 6sec
					open(LCK, ">$user_lock");
					print LCK "!\n";
					close LCK;
					$step_next= 8; 
					$err_msg= $messages{$lang}[18];
					sleep 6;
					unlink $user_lock;
				}
			} else {
				# database error
				$step_next= 101; 
				$err_msg= $messages{$lang}[5];
			}
			$mysql -> disconnect ();
		} else { 
			$step_next= 8; 
			$err_msg= $messages{$lang}[2]; 
		}
	} else  { 
		$step_next= 100; 
		$err_msg= $messages{$lang}[0]; 
	}
}

elsif ( $step_akt == 11 ) {
	# user send change contact request
	if (defined($user_login)) {
		my $kontakt= $query->param('kontakt');
		if(defined($kontakt)) {
			if($kontakt eq '') {
				# delete the old contact info
				my $cfile= $main_dir . $user_login . '/contact.txt.asc';
				if(-f $cfile) { 
					system("shred -n 2 -u $cfile"); 
					$info_msg= $messages{$lang}[19];
				} else {
					$info_msg= $messages{$lang}[20];
				}
			} else {
				# update the contact info
				my $gnupg= GnuPG::Interface->new();
				$gnupg->options->hash_init( armor   => 1, homedir => $admin_key_dir, always_trust => 0);
			   $gnupg->options->push_recipients( @admin_keys );
				$gnupg->options->meta_interactive(0);

				my $rein= IO::Handle->new();
				my $raus= IO::Handle->new();
				my $rerr= IO::Handle->new();
				my $rr= GnuPG::Handles->new(stdin => $rein, stdout => $raus, stderr => $rerr );

				my $gpg_pid= $gnupg->encrypt( handles => $rr );
				print $rein $kontakt, "\n";
				close $rein;
				my @cipher;
				if(defined($raus)) { @cipher= <$raus>; }
				close $raus;
				close $rerr;
				waitpid $gpg_pid, 0;

				my $userconfig= $main_dir . $user_login . '/contact.txt.asc';
				open(OUTF, ">$userconfig");
				print OUTF @cipher, "\n";
				close OUTF;
				$info_msg= $messages{$lang}[21];
			}
			$step_next= 100; 
		} else { 
			$step_next= 8; 
			$err_msg= $messages{$lang}[2]; 
		}
	} else  { 
		$step_next= 100; 
		$err_msg= $messages{$lang}[0]; 
	}
}

elsif ( $step_akt == 13 ) {
	# Nachricht an Admin-Team
	if (defined($user_login)) {
		my $subj= $query->param('betreff');
		$text_admin= $query->param('artikel');
		if((defined($subj)) && (defined($text_admin)) && ($subj ne '') && ($text_admin ne '')) {

			# Einlesen der Admin-Einstellungen
			my $config= $main_dir . 'admin/user.cfg';
			open(CFG, $config);
			my $admin_configuration= <CFG>;
			chomp $admin_configuration;
			close CFG;
			my $adminhashref = $cfo->decrypt($admin_configuration);
			my $forward_admin= $adminhashref->{'fwd'};
			my $email_admin= $adminhashref->{'email'};
			my $pgp_id_admin= $adminhashref->{'pgp'};

			# Nachricht zusammenbauen
			my $message= 'From: ' . $from_name . ' <' . $from_address . '>' . "\n";
			my $datum= time2str("%a, %e %b %Y %T %z", time() );
			$message= $message . "Date: $datum\n";
			$message= $message . "User-Agent: PrivacyBox 1.0\nMIME-Version: 1.0\n";
			$message= $message . "Content-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\n";
			$message= $message . "Subject: $subj \n";
			if ($forward_admin eq '2') { 
				$message= $message . "To: $email_admin\n"; 
				$message= $message . 'Reply-To: ' . $reply_name . ' <' . $reply_address . '>' . "\n\n";
			} else { 
				$message= $message . 'To: admin@' . $self_host . "\n"; 
				$message= $message . 'Reply-To: ' . $reply_name . ' <' . $reply_address . '>' . "\n\n";
			}

			# Nachricht verschlüsseln
			my $gnupgpath_admin= $main_dir . 'admin';
			my $gnupg= GnuPG::Interface->new();
			$gnupg->options->hash_init( armor   => 1, homedir => $gnupgpath_admin, always_trust => 1 );
			$gnupg->options->push_recipients( $pgp_id_admin ); 
			$gnupg->options->meta_interactive(0);

			my $rein= IO::Handle->new();
			my $raus= IO::Handle->new();
			my $rerr= IO::Handle->new();
			my $rr= GnuPG::Handles->new(stdin => $rein, stdout => $raus, stderr => $rerr );

			my $gpg_pid= $gnupg->encrypt( handles => $rr );
			print $rein "Anfrage von PrivacyBox User: $user_login \n\n";
			print $rein $text_admin;
			close $rein;
			my @errortext;
			if(defined($rerr)) { @errortext= <$rerr>; }
			my $cipherline;
			if(defined($raus)) { $cipherline= <$raus>; }
			$message= $message . $cipherline;
			$message= $message . "Charset: UTF-8\n";
			while ((defined($raus)) && ($cipherline= <$raus>) && ($cipherline =~ /^[A-Za-z]/ )) { };
			if($cipherline) {
				$message= $message . $cipherline;
				while ($cipherline= <$raus>) { $message= $message . $cipherline; }
			}
			close $raus;
			close $rerr;
			waitpid $gpg_pid, 0;
			if ( @errortext > 0) { $err_msg= $messages{$lang}[22]; } 
			else {
				# Nachricht senden
				if ($forward_admin eq "1") {
					# save message local
					my $random_msg= int(rand 120000);
					my $mailmsg= $mail_dir . 'admin/Maildir/new/' . $random_msg . '@' . $self_host;
					while(-f $mailmsg) {
						$random_msg= int(rand 120000);
						$mailmsg= $mail_dir . 'admin/Maildir/new/' . $random_msg . '@' . $self_host;
					}
					open(MSG, ">$mailmsg");
					if (MSG) {
						print MSG $message;
						close MSG;
						$info_msg= $messages{$lang}[23];
					} else { 
						$err_msg= $messages{$lang}[22];
					}
				} else  {
					# send message to an extern mail account
					my $mailer = Email::Send->new({mailer => 'SMTP'});
					$mailer->mailer_args([Host => "$mailhost:$mailport", ssl => $mailssl]);
					if (! $mailer->send($message)) { $info_msg= $messages{$lang}[23];}
					else {$err_msg= $messages{$lang}[22]; }
				}
			}
			$step_next= 100;
		} else { 
			$step_next= 12; 
			$err_msg= $messages{$lang}[24]; 
		}
	} else {
		$step_next= 100; 
		$err_msg= $messages{$lang}[0];
	}
}

elsif ( $step_akt == 17 ) {
	# löschen des S/MIME Zertifikates
	if (defined($user_login)) {
		# Prüfen der Einstellungen
		my $config= $main_dir . $user_login . '/user.cfg';
		open(CFG, $config);
		my $user_configuration= <CFG>;
		chomp $user_configuration;
		close CFG;
		my $newhashref = $cfo->decrypt($user_configuration);
		if ($newhashref->{'encr'} eq "2") {
			# Deaktivieren der Verschlüsselung
			$newhashref->{'encr'}= "0";
			$user_configuration= $cfo->encrypt($newhashref);
			open(CFG, ">$config");
			print CFG $user_configuration, "\n";
			close CFG;
			$err_msg= $messages{$lang}[25];
		}
		my $cert= $main_dir . $user_login . '/smime_tmp.pem';
		if(-f $cert) { system("shred -n 2 -u $cert"); }
		$cert= $main_dir . $user_login . '/smime.pem';
		if(-f $cert) { system("shred -n 2 -u $cert"); }
		$step_next= 16;
	} else {
		$step_next= 100; 
		$err_msg= $messages{$lang}[0];
	}
}

elsif ( $step_akt == 19 ) {
	# löschen der PGP Schlüssel
	if (defined($user_login)) {
		# Prüfen der Einstellungen
		my $config= $main_dir . $user_login . '/user.cfg';
		open(CFG, $config);
		my $user_configuration= <CFG>;
		chomp $user_configuration;
		close CFG;
		my $newhashref = $cfo->decrypt($user_configuration);
		if ($newhashref->{'encr'} eq "1") {
			# Deaktivieren der Verschlüsselung
			$newhashref->{'encr'}= "0";
			$user_configuration= $cfo->encrypt($newhashref);
			open(CFG, ">$config");
			print CFG $user_configuration, "\n";
			close CFG;
			$err_msg= $messages{$lang}[25];
		}
		my $file= $main_dir . $user_login . '/pubring.gpg';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$file= $main_dir . $user_login . '/pubring.gpg~';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$file= $main_dir . $user_login . '/trustdb.gpg';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$file= $main_dir . $user_login . '/secring.gpg';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$file= $main_dir . $user_login . '/openpgp.asc';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$file= $main_dir . $user_login . '/openpgp_mini.asc';
		if(-f $file) { system("shred -n 2 -u $file"); }
		$step_next= 16;
	} else {
		$step_next= 100; 
		$err_msg= $messages{$lang}[0];
	}
}

elsif ( $step_akt == 23 ) {
	# Account kündigen
	if (defined($user_login)) { 
		my $passwd= $query->param('pw0');
		if((defined($passwd)) && ($passwd ne '')) {
			# check the password
			my $user_lock= $main_dir . $user_login . '/lock.txt';
			while(-f $user_lock) { sleep 7; }
			my $mysql = DBI -> connect ("DBI:mysql:database=privacybox", $mysql_user, $mysql_passwd, {'RaiseError' => 1});
			my $statement = $mysql -> prepare ("SELECT * FROM passwd WHERE username = ?");
			$statement -> execute ($user_login);
			if (my $row = $statement -> fetchrow_hashref ()) {	
				my $passwd_saved= $row -> {'passwd_crypt'};
				$statement->finish ();
				my @pa= split //, $passwd_saved;
				my $salt2= $pa[0] . $pa[1];
				if(  crypt($passwd, $salt2) eq $passwd_saved) {
					# everything is ok, delete the account
					rmtree( $main_dir . $user_login ,0,1);
					rmtree( $mail_dir . $user_login ,0,1);
					$statement = $mysql -> prepare ("DELETE FROM passwd WHERE username = ?");
					$statement -> execute ($user_login);
					$statement->finish ();
					$mysql -> disconnect ();
					$user_login= undef;
					$info_msg= $messages{$lang}[26];
					$step_next= 100;
				} else {
					# wrong password, lock the account for 6sec
					$mysql -> disconnect ();
					open(LCK, ">$user_lock");
					print LCK "!\n";
					close LCK;
					$step_next= 101; 
					$err_msg= $messages{$lang}[18];
					sleep 6;
					unlink $user_lock;
				}
			} else {
				# database error
				$mysql -> disconnect ();
				$step_next= 101; 
				$err_msg= $messages{$lang}[5];
			}
		} else { 
			$step_next= 22; 
			$err_msg= $messages{$lang}[2]; 
		}
	} else  { 
		$step_next= 100; 
		$err_msg= $messages{$lang}[0]; 
	}
}

# nothing to do
else {  $step_next= $step_akt; }

if ($step_next == 4) { $step_akt=4; }

#####################################################
# Set the default url and add the session id,
# save the session status
#####################################################
my $url= $self . '?lang=' . $lang;
if(defined($user_login)) { 

	my $lts= time();
	my $sid= int(rand(12000));

	my $ahashref = { lts => $lts, sid => $sid };
	if(defined($session_ip)) { $ahashref->{'ip'} = $session_ip; }
	my $session_save= $cfo->encrypt($ahashref);
	my $sessionfile= $main_dir . $user_login . '/session.txt';
	open(SES, ">$sessionfile");
	print SES $session_save, "\n";
	close SES;

	my $bhashref = { uid => $user_login, sid => $sid };
	$session= $cfo->encrypt($bhashref);
	$url= $url .'&amp;se=' . $session;
} else { $session= undef; }

#######################################
# Write the header of the new HTML page
#######################################
print "Content-type: text/html\n\n";
print '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"><html>';
print '<head><title>PrivacyBox</title>  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">';
print '<meta http-equiv="expires" content="0">';
print '<meta NAME="robots" content="noindex,nofollow"><meta http-equiv="Content-Style-Type" content="text/css">';
print '<LINK rel="StyleSheet" type="text/css" href="/pbox.css">';
print '</head><body><div id="all">';

print '<a href="http://www.wirspeichernnicht.de/content/view/10/22/"><IMG src="/images/wirspeichernnichtsiegel_d.png" width="109" height="113" align="right" border="0"></a>';

print '<div id="head"><h1>PrivacyBox</h1></div>';

print '<div id="language_selection">';
print '<a href="', $self, '?ac=', $step_akt, '&amp;lang=en';
if (defined($session)) { print '&amp;se=', $session; }
print '">';
print '<img src="/images/en.png" hspace="10" vspace="0" alt="english" width="24" height="16" border="0"></a>';
print '<a href="', $self, '?ac=', $step_akt, '&amp;lang=de';
if (defined($session)) { print '&amp;se=', $session; }
print '">';
print '<img src="/images/de.png" hspace="5" vspace="0" alt="deutsche" width="24" height="16" border="0"></a>';
print '<a href="', $self, '?ac=', $step_akt, '&amp;lang=fr';
if (defined($session)) { print '&amp;se=', $session; }
print '">';
print '<img src="/images/fr.png" hspace="10" vspace="0" alt="fran&ccedil;ais" width="24" height="16" border="0"></a>';
print '</div>';


if(defined($user_login)) { 
	print '<div id="anonymous_use">', $messages{$lang}[27], '<div id="name">', $user_login, '</div></div>'; 
} else {
	print '<div id="anonymous_use">';
	if($lang eq 'de') {
		print 'Sie können die PrivacyBox anonym und unbeobachtet mit '; 
		if ($conntyp == 3) { print '<span class="anonymous_active">JonDonym</span>'; }
		else { print '<a href="https://www.jondos.de/de/" class="anonymous_use_link" target="_blank">JonDonym</a>'; }
		print ' nutzen.<br> Die PrivacyBox ist auch als ';
		if ($conntyp == 11 ) { print '<span class="anonymous_active">Tor&nbsp;Hidden&nbsp;Service</span>'; } 
		else {
			print '<a href="http://', $onion_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=de" class="anonymous_use_link">Tor&nbsp;Hidden&nbsp;Service</a>';
		}
		print ' und ';
		if ($conntyp == 12 ) { print '<span class="anonymous_active">I2P&nbsp;eepsite</span>'; } 
		else {
			print '<a href="http://', $i2p_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=de" class="anonymous_use_link">I2P&nbsp;eepsite</a>';
		}
		print ' erreichbar.';
	} elsif ($lang eq 'fr') {
		# Der Hinweis auf die Hidden Services in Französisch
		print 'Nous vous conseillons d&apos;utiliser ce service, anonyme et intra&ccedil;able, par ';
		if ($conntyp == 3) { print '<span class="anonymous_active">JonDonym</span>'; }
		else { print '<a href="https://www.jondos.de/en/" class="anonymous_use_link" target="_blank">JonDonym</a>'; }
		print ', par ';
		if ($conntyp == 11 ) { 
			print '<span class="anonymous_active">l&apos;interm&eacute;diaire&nbsp;du&nbsp;service&nbsp;TOR</span>';
		} else {
			print '<a href="http://', $onion_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=fr" class="anonymous_use_link">l&apos;interm&eacute;diaire&nbsp;du&nbsp;service&nbsp;TOR</a>';
		}
		print ', en ';
		if ($conntyp == 12 ) { 
			print '<span class="anonymous_active">tant&nbsp;que&nbsp;site&nbsp;I2P</span>';
		} else {
			print '<a href="http://', $i2p_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=fr" class="anonymous_use_link">tant&nbsp;que&nbsp;site&nbsp;I2P</a>';
		}
		print ' ou avec un <a href="https://www.awxcnx.de/cgi-bin/proxy4/nph-proxy.cgi/000000A/http/privacybox.i2p/cgi-bin/tram_login.pl?ac=0&amp;lang=fr" class="anonymous_use_link">proxy&nbsp;web&nbsp;I2P</a>.';

	} else {
		# Englischer Hinweis auf Hidden
		print 'You may use the PrivacyBox anonymous and not traceable by ';
		if ($conntyp == 3) { print '<span class="anonymous_active">JonDonym</span>'; }
		else { print ' <a href="https://www.jondos.de/en/" class="anonymous_use_link">JonDonym</a>'; }
		print '.<br>';
		print 'The PrivacyBox ist accessible by ';
		if ($conntyp == 11 ) { print '<span class="anonymous_active">Tor&nbsp;hidden&nbsp;service</span>'; } 
		else {
			print '<a href="http://', $onion_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=en" class="anonymous_use_link">Tor&nbsp;hidden&nbsp;service</a>';
		}
		print ' or ';
		if ($conntyp == 12 ) { print '<span class="anonymous_active">I2P&nbsp;eepsite</span>'; } 
		else {
			print '<a href="http://', $i2p_host, '/cgi-bin/tram_login.pl?ac=0&amp;lang=en" class="anonymous_use_link">I2P&nbsp;eepsite</a>';
		}
		print ' too.';
	}
	print '</div>';
}

print '<div id="wirspeichernnichtlogo"><a href="http://www.wirspeichernnicht.de/content/view/10/22/">';
print '<IMG src="/images/wirspeichernnichtsiegel_d.png" width="109" height="113" border="0"></a></div>';

# navigation toolbar
print '<div id="navigation"><div id="navigation_pbox">';
if(defined($user_login)) { 
	# write the navigation bar for user logged in
	if($step_next == 4) { print '<span class="navigation_pbox_active">', $messages{$lang}[28], '</span> &nbsp; '; }
	else {  print '<a href="', $url, '&amp;ac=4" class="navigation_pbox_link">', $messages{$lang}[28], '</a> &nbsp; '; }
	if($step_next == 8) { print '<span class="navigation_pbox_active">', $messages{$lang}[29], '</span> &nbsp; '; }
	else {  print '<a href="', $url, '&amp;ac=8" class="navigation_pbox_link">', $messages{$lang}[29], '</a> &nbsp; '; }
	if($step_next == 16) { print '<span class="navigation_pbox_active">', $messages{$lang}[30], '</span> &nbsp; '; }
	else {  print '<a href="', $url, '&amp;ac=16" class="navigation_pbox_link">', $messages{$lang}[30], '</a> &nbsp; '; }
	if($step_next == 22) { print '<span class="navigation_pbox_active">', $messages{$lang}[31], '</span> &nbsp; '; }
	else {  print '<a href="', $url, '&amp;ac=22" class="navigation_pbox_link">', $messages{$lang}[31], '</a> &nbsp; '; }
	if($step_next == 12) { print '<span class="navigation_pbox_active">', $messages{$lang}[32], '</span> &nbsp; '; }
	else {  print '<a href="', $url, '&amp;ac=12" class="navigation_pbox_link">', $messages{$lang}[32], '</a>'; }
	print '</div><div id="navigation_webservices">';
	print '<a href="', $url, '&amp;ac=2" class="navigation_webservices_link"><strong>', $messages{$lang}[33], '</strong></a>';
	print '</div>';
} else {
	# write the navigation bar for user not logged in
	print '<a href="/index.', $lang,'.html" class="navigation_pbox_link">';
	print $messages{$lang}[34], '</a> &nbsp; ';
	print '<a href="/cgi-bin/tram_neu.pl?lang=', $lang,'&amp;ac=0" class="navigation_pbox_link">';
	print $messages{$lang}[35], '</a> &nbsp; ';
	if ( $step_next == 0 ) { print '<span class="navigation_pbox_active">', $messages{$lang}[36], '</span> &nbsp; '; }
	else {  print '<a href="', $self, '?ac=0&lang=', $lang, '" class="navigation_pbox_link">', $messages{$lang}[36], '</a> &nbsp; '; }
	print '&nbsp;&nbsp;<a href="/cgi-bin/tram_msg.pl?sus=admin&amp;lang=', $lang, '" class="navigation_pbox_link">', $messages{$lang}[37], '</a>';
	# Webservices
	print '</div><div id="navigation_webservices">';
	print '<a href="/tor-proxy.', $lang ,'.html" class="navigation_webservices_link">', $messages{$lang}[85], '</a> &nbsp; ';
	print '<a href="/i2p-proxy.', $lang ,'.html" class="navigation_webservices_link">', $messages{$lang}[86], '</a>';
	print '</div>';
}
print '</div>';


#######################################
# Write the content of the new HTML page
#
# depending on $next_step:
#  0: login form
#  3: FAQ
#  4: config settings form
#  8: change password /contact info
# 12: admin kontakt form
# 16: Public Keys
# 22: Kündigung
#######################################
print '<div id="content">';

# print info/error message
if ( defined($info_msg) ) { print '<div class="info">', $info_msg, '</div>'; }
if ( defined($err_msg) ) { print '<div class="error">', $err_msg, '</div>'; }

if ($step_next == 0 ) {
	# login form
	print '<br><br><FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="1">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table><tr><td align="right">', $messages{$lang}[39], ':</td><td align="left">';
	print '<INPUT type="text" name="us"></td></tr>';
	print '<tr><td align="right">', $messages{$lang}[40], ':</td><td align="left">';
	print '<INPUT type="password" name="pw"></td></tr>';
	if ($conntyp == 0 ) {
		print '<tr><td>&nbsp;</td><td align="left"><INPUT type="checkbox" checked name="ip" value="1">&nbsp;';
		print $messages{$lang}[84];
		print '</td></tr>';
	}
	print '<tr><td></td><td align="left"><br><INPUT type="submit" value="', $messages{$lang}[36], '"></td></tr></table></div><FORM>';
}


elsif ( $step_next == 4 ) {
	# read old config values
	my $config= $main_dir . $user_login . '/user.cfg';
	open(CFG, $config);
	my $user_configuration= <CFG>;
	chomp $user_configuration;
	close CFG;
	my $newhashref = $cfo->decrypt($user_configuration);

	# write the formular
	if (!(defined($err_msg))) {
		print '<div class="info">', $messages{$lang}[41], ' <a href="', $url, '&ac=12">', $messages{$lang}[42], '</a>.';
	}
	print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="5">';
	print '<INPUT type="hidden" name="se" value="', $session, '">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table>';
	print '<tr><td colspan="4" align="left"><h2>', $messages{$lang}[81], '</h2></td></tr>';
	print '<tr><td><INPUT type="radio" name="la" value="en"';
	if ($lang eq 'en') { print ' checked'; }
	print '></td><td align="left"><img src="/images/en.png" hspace="10" vspace="0" alt="english" width="24" height="16" border="0"></td>';
	print '<td align="left"><INPUT type="radio" name="la" value="de"';
	if ($lang eq 'de') { print ' checked'; }
	print '><img src="/images/de.png" hspace="10" vspace="0" alt="deutsch" width="24" height="16" border="0"></td>';
	print '<td align="left"><INPUT type="radio" name="la" value="fr"';
	if ($lang eq 'fr') { print ' checked'; }
	print '><img src="/images/fr.png" hspace="10" vspace="0" alt="fran&ccedil;ais" width="24" height="16" border="0"></td>';
	print '<tr>';

	print '<tr><td colspan="4" align="left"><h2>', $messages{$lang}[43], '</h2></td></tr>';
	print '<tr><td>&nbsp;</td><td  align="left">&nbsp;&nbsp;', $messages{$lang}[44], ':</td><td colspan="2" align="left"><a href="https://', $self_host, '/', $user_login, '.msg" target="_blank">https://privacybox.de/', $user_login, '.msg</a></td></tr>';
	print '<tr><td>&nbsp;</td><td  align="left">&nbsp;&nbsp;', $messages{$lang}[45], ':</td><td colspan="2" align="left"><a href="https://', $self_host, '/', $user_login, '.mobi" target="_blank">https://privacybox.de/', $user_login, '.mobi</a></td></tr>';
	print '<tr><td>&nbsp;</td><td  align="left">&nbsp;&nbsp;', $messages{$lang}[46], ':</td><td colspan="2" align="left"><a href="http://', $onion_host, '/', $user_login, '.msg" target="_blank">http://', $onion_host, '/', $user_login, '.msg</a></td></tr>';
	print '<tr><td>&nbsp;</td><td  align="left">&nbsp;&nbsp;', $messages{$lang}[80], ':</td><td colspan="2" align="left"><a href="http://', $i2p_host, '/', $user_login, '.msg" target="_blank">http://', $i2p_host, '/', $user_login, '.msg</a></td></tr>';

	print '<tr><td colspan="4" align="left"><h2>', $messages{$lang}[47], '</h2></td></tr>';
	print '<tr><td valign="top"><INPUT type="radio" name="fw" value="1"';
	if ($newhashref->{'fwd'} eq "1") { print ' checked'; }
	print '></td><td colspan="3" align="left">', $messages{$lang}[48], '</td></tr>';
	print '<tr><td>&nbsp;</td><td align="left">&nbsp;&nbsp;', $messages{$lang}[49], ': </td><td align="left"> ', $self_host, ' </td><td align="left">&nbsp;&nbsp;', $messages{$lang}[50], ': 995 (SSL)</td></tr>';
	print '<tr><td>&nbsp;</td><td align="left">&nbsp;&nbsp;', $messages{$lang}[46], ':&nbsp;&nbsp;&nbsp;</td><td align="left"> ', $onion_host, ' </td><td align="left">&nbsp;&nbsp;', $messages{$lang}[50], ': 110</td></tr>';
	print '<tr><td colspan="4">&nbsp;</td></tr>';
	print '<tr><td valign="top"><INPUT type="radio" name="fw" value="2"';
	if ($newhashref->{'fwd'} eq "2") { print ' checked'; }
	print '><td colspan="3" align="left">', $messages{$lang}[51], '</td></tr>';
	print '<tr><td>&nbsp;</td><td align="right">&nbsp;&nbsp;', $messages{$lang}[52], ': </td><td align="left" colspan="2"><INPUT type="text" name="em" size="30"';
	if ($newhashref->{'email'} ne "") { print '" value="', $newhashref->{'email'}, '"'; }
	print '></td></tr>';
	print '<tr><td colspan="4">&nbsp;</td></tr>';
	print '<tr><td valign="top"><INPUT type="radio" name="fw" value="3"';
	if ($newhashref->{'fwd'} eq "3") { print ' checked'; }
	print '><td colspan="3" align="left">', $messages{$lang}[82], '</td></tr>';
	print '<tr><td>&nbsp;</td><td align="right">&nbsp;&nbsp;', $messages{$lang}[83], ': </td><td align="left" colspan="2"><INPUT type="text" name="i2p" size="30"';
	if ((defined($newhashref->{'i2p'})) && ($newhashref->{'i2p'} ne "")) { print '" value="', $newhashref->{'i2p'}, '"'; }
	print '>@mail.i2p</td></tr>';

	print '<tr><td colspan="4" align="left"><h2>', $messages{$lang}[53], '</h2></td></tr>';
	print '<tr><td><INPUT type="radio" name="encr" value="0"';
	if ($newhashref->{'encr'} eq "0") { print ' checked'; }
	print '></td><td colspan="3" align="left">', $messages{$lang}[54], '</td></tr>';
	print '<tr><td><INPUT type="radio" name="encr" value="2"';
	if ($newhashref->{'encr'} eq "2") { print ' checked'; }
	print '></td><td colspan="3" align="left">', $messages{$lang}[55], '</td></tr>';
	print '<tr><td><INPUT type="radio" name="encr" value="1"';
	if ($newhashref->{'encr'} eq "1") { print ' checked'; }
	print '></td><td colspan="3" align="left">', $messages{$lang}[56], '</td></tr>';
	print '<tr><td>&nbsp;</td><td align="right">&nbsp;&nbsp;', $messages{$lang}[57], ': </td><td align="left" colspan="2"><INPUT type="text" name="pgp" size="30%"';
	if ($newhashref->{'pgp'} ne "") { print '" value="', $newhashref->{'pgp'}, '"'; }
	print '></td></tr><tr><td colspan="2"></td><td colspan="2" align="left">(', $messages{$lang}[58], ')</td></tr>';

	print '<tr><td colspan="4" align="left"><h2>', $messages{$lang}[59], '</h2></td></tr>';
	print '<tr><td colspan="4">', $messages{$lang}[60], '</td></tr>';
	print '<td>&nbsp;</td><td align="right">', $messages{$lang}[61], ': </td><td align="left" colspan="2"><input type="file" name="key" accept="*/*" maxlength="56600"></td></tr>';

	print '<tr><td colspan="4"><br><INPUT type="submit" value="', $messages{$lang}[62], '"><br><br><br></td></tr>';
	print '</div></table></FORM>';
}

elsif ($step_next == 8 ) {
	print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="9">';
	print '<INPUT type="hidden" name="se" value="', $session, '">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table><tr><td colspan="2"><h2>', $messages{$lang}[63], '</h2></td></tr><tr><td align="right">';
	print $messages{$lang}[64], ': ';
	print '</td><td align="left"><INPUT type="password" name="pw0"></td></tr><tr><td colspan="2">&nbsp;</td></tr><tr><td align="right">';
	print $messages{$lang}[65], ': ';
	print '</td><td align="left"><INPUT type="password" name="pw1"></td></tr><tr><td align="right">';
	print $messages{$lang}[65], ': ';
	print '</td><td align="left"><INPUT type="password" name="pw2"></td></tr>';
	print '<tr><td></td><td align="left"><INPUT type="submit" value="';
	print '&nbsp;&nbsp;', $messages{$lang}[63], '&nbsp;&nbsp;';
	print '"></td></tr></table></div></FORM>';

	print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="11">';
	print '<INPUT type="hidden" name="se" value="', $session, '">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table><tr><td colspan="2"><h2>', $messages{$lang}[66], '</h2></td></tr><tr><td align="right">';
	print $messages{$lang}[67], ': ';
	print '</td><td align="left"><INPUT type="password" name="kontakt"></td></tr>';
	print '<tr><td></td><td align="left"><INPUT type="submit" value="', $messages{$lang}[66], '"></td></tr></table></div></FORM>';
}

elsif ($step_next == 12 ) {
	print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="13">';
	print '<INPUT type="hidden" name="se" value="', $session, '">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table><tr><td align="right">', $messages{$lang}[68], ': </td>';
	print '<td align="left">&nbsp;<strong>', $messages{$lang}[69], '</strong></td>';
	print '</tr><tr><td align="right">', $messages{$lang}[70], ': </td>';
	print '<td align="left"><input type="text" name="betreff" tabindex="1" size="35%"></td>';
	print '</tr><tr><td colspan="2"><br><textarea rows="10" cols="70" name="artikel" tabindex="2">', $text_admin, '</textarea>';
	print '</td></tr></table><br>';
	print '<input type="submit" VALUE="', $messages{$lang}[71], '"><br><br><br></div></form>';
}

elsif ($step_next == 16 ) {
	print '<div align="center"><table><tr><td align="left">';
	my $userdir= $main_dir . $user_login;
	# check for S/MIME zertificates
	my $cert_a= $userdir . '/smime.pem';
	my $cert_b= $userdir . '/smime_tmp.pem';
	my $cert= 0;
	if((-f $cert_a) || (-f $cert_b)) {
		print '<h2>', $messages{$lang}[72], '</h2>';
		print $messages{$lang}[73], '<br>';
		print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
		print '<INPUT type="hidden" name="ac" value="17">';
		print '<INPUT type="hidden" name="se" value="', $session, '">';
		print '<INPUT type="hidden" name="lang" value="', $lang, '">';
		print '<input type="submit" VALUE="', $messages{$lang}[74], '"></form>';
		$cert= 1;
	}
	# check for PGP keys
	my $gnupg= GnuPG::Interface->new();
	$gnupg->options->hash_init( armor   => 1, homedir => $userdir, always_trust => 1 );
	$gnupg->options->meta_interactive(0);
	my $rein= IO::Handle->new();
	my $raus= IO::Handle->new();
	my $rerr= IO::Handle->new();
	my $rr= GnuPG::Handles->new(stdin => $rein, stdout => $raus, stderr => $rerr );
	my $gpg_pid = $gnupg->wrap_call( 
		commands => [ qw(--list-keys) ],
		handles => $rr 
	);
	close $rein;
	my @errortext;
	if($rerr) { @errortext= <$rerr>; }
	my @output;
	if(defined($raus)) { @output= <$raus>; }
	close $raus;
	close $rerr;
	waitpid $gpg_pid, 0;
	if ( @output > 2) {
		print '<h2>', $messages{$lang}[75], '</h2>';
		for(my $i=2; $i<@output; $i++) {
			my $line= $output[$i];
			chomp($line);
			print $line, '<br>';
		}
		print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
		print '<INPUT type="hidden" name="ac" value="19">';
		print '<INPUT type="hidden" name="se" value="', $session, '">';
		print '<INPUT type="hidden" name="lang" value="', $lang, '">';
		print '<input type="submit" VALUE="', $messages{$lang}[76], '"></form>';
	} else {
		if($cert == 0) { print $messages{$lang}[77]; }
	}
	print '</td></tr></table></div>';
}

elsif ($step_next == 22 ) {
	if (!(defined($info_msg))) {
		print '<div class="info">', $messages{$lang}[78], '</div>';
	}
	print '<FORM action="', $self, '" method="POST" enctype="multipart/form-data">';
	print '<INPUT type="hidden" name="ac" value="23">';
	print '<INPUT type="hidden" name="se" value="',  $session, '">';
	print '<INPUT type="hidden" name="lang" value="', $lang, '">';
	print '<div align="center"><table><tr><td align="right">';
	print $messages{$lang}[40];
	print '</td><td align="left"><INPUT type="password" name="pw0"></td></tr>';
	print '<tr><td></td><td align="left"><INPUT type="submit" value="', $messages{$lang}[79], '">';
	print '</td></tr></table></div></FORM>';
}

print '</div></div></body></html>';
